import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.widgets import Slider, TextBox

import math
import pickle

cote = 1 # taille du côté d'une case
rayon = 8 # nombre de cases par côté
diametre = 2*rayon+1 # nombre de cases au total dans la longueur ou largeur
texte_max = 441 # Le dernier nombre affiché avec du texte
bords_max = 16  # L'étape après laquelle on n'affiche plus les murs
XMAX = (rayon+.5)*cote  # Le maximum pour l'abscisse de la fenêtre
taille_point = 15 # La taille des points avec le zoom maximum

coeff = 1.10 # on augmente de 10% à chaque étape
t_max = 20 # le nombre maximum d'étapes (on peut aller jusqu'à 30)
DEBUT = 1 # Le nombre au centre
debut_max = 100 # on peut aller jusqu'à mettre 100 au centre


fig, ax = plt.subplots()
fig.set_figheight(7) # La fenêtre fait 7 pouces de large
fig.set_figwidth(7) # La fenêtre fait 7 pouces de haut
ax.axis(False) # On n'affiche pas les axes

ax.set_ylim(-XMAX, XMAX)  # On définit les valeurs maximales
ax.set_xlim(-XMAX, XMAX)  # pour le repère affiché.
ax.set_aspect('equal', adjustable='box')  # Repère orthonormé

# On calcule le nombre maximum de valeurs à afficher
nmax = (1+2*round(rayon*(coeff**t_max)))**2

# Pour faire la spirale
directions = [(1, 0), (0, 1), (-1, 0), (0, -1)]

# au cas où on n'a pas de fichier de nombres premiers
"""
liste_premiers = [2]

def premiers_premiers(n, liste_premiers):
    for i in range(3, n+1, 2):
        if est_premier(i, liste_premiers):
            liste_premiers.append(i)

def est_premier(n, liste_premiers):
    racine = int(math.sqrt(n))
    i = 0
    while liste_premiers[i] <= racine:
        if n % liste_premiers[i] == 0:
            return False
        i += 1
    return True
    
premiers_premiers(int(nmax)+1, liste_premiers)
"""
def lit_premiers(nom_fichier="liste_premiers.dat"):
    with open(nom_fichier, "rb") as fichier:
        return pickle.load(fichier)

# on ne garde que les nombres premiers nécessaires
liste_premiers = [p for p in lit_premiers("liste_premiers_500millions.dat") if p <= nmax]
print(f"On a {len(liste_premiers)} nombres premiers dont le plus grand est {liste_premiers[-1]}")

# On calcule les positions de toutes les cases,
# on dessine les murs et on affiche les textes
def faire_spirale(nmax):
    n = 1
    pas = 1
    d = 0
    x, y = 0, 0
    texte = plt.text(x, y, str(n), ha='center', va='center')
    liste_textes = [texte]
    liste_positions = [(x, y)]
    xmurs = [x+cote/2, x-cote/2, x-cote/2]
    ymurs = [y+cote/2, y+cote/2, y-cote/2]
    while n < nmax:
        dx, dy = directions[d]
        xmurs.append(xmurs[-1]+cote*dx)
        ymurs.append(ymurs[-1]+cote*dy)
        for i in range(int(pas)):
            if n < nmax:
                n += 1
                x += cote*dx
                y += cote*dy
                if n <= texte_max:
                    texte = plt.text(x, y, str(n), ha='center', va='center')
                    liste_textes.append(texte)
                xmurs.append(xmurs[-1]+cote*dx)
                ymurs.append(ymurs[-1]+cote*dy)
                liste_positions.append((x, y))
        pas += 0.5
        d = (d+1)%4
    bords, = plt.plot(xmurs, ymurs)
    return liste_textes, bords, liste_positions

liste_textes, bords, liste_positions = faire_spirale(nmax)

# On place des cercles à chaque nombre premier
def placer_premiers():
    liste_cercles = []
    for p in liste_premiers:
        if p-1 < len(liste_positions):
            x, y = liste_positions[p-1]
            visible = True
        else:
            x, y = 0, 0
            visible = False
        cercle = patches.Circle((x, y), 0.45*cote, color="red", alpha=.5, visible=visible)
        liste_cercles.append(cercle)
        ax.add_patch(cercle)
    return liste_cercles

liste_cercles = placer_premiers()

# Pour afficher l'équation de droite
droite, = ax.plot([], [], 'gs', alpha=.4, markersize=taille_point);

equation_droite = ""  # On commence sans équation

# Pour calculer les points de la "droite"
def mettre_a_jour_droite(equation):
    global equation_droite
    equation_droite = equation
    xdroite = []
    ydroite = []
    if equation_droite != "":
        n = 0
        try:
            print("on va évaluer", equation_droite)
            n2 = eval(equation_droite)
            print(n2)
            while n2-DEBUT < nmax:
                if 0 <= n2-DEBUT < nmax:
                    x, y = liste_positions[n2-DEBUT]
                    xdroite.append(x)
                    ydroite.append(y)
                n += 1
                n2 = eval(equation_droite)
        except: # 
            print("L'équation", equation_droite, "n'est pas correcte")
    droite.set_data(xdroite, ydroite)
        
mettre_a_jour_droite(equation_droite)

# On met à jour à l'étape t
def mise_a_jour(t):
    c = coeff**t
    r = round(rayon*c)
    nmax = (1+2*r)**2
    xmax = (r+.5)*cote
    ax.set_ylim(-xmax, xmax)
    ax.set_xlim(-xmax, xmax)
    for i in range(len(liste_textes)):
        liste_textes[i].set_visible(nmax <= texte_max and i < nmax)
    bords.set_visible(t <= bords_max)
    droite.set_markersize(max(taille_point/c, 1))

# On change la valeur du début
def mise_a_jour_debut(debut):
    global DEBUT
    DEBUT = debut
    for i in range(len(liste_textes)):
        liste_textes[i].set_text(str(i+debut))
    for j in range(len(liste_premiers)):
        p = liste_premiers[j]
        if 0 <= p-debut < nmax:
            liste_cercles[j].set_visible(True)
            liste_cercles[j].set_center(liste_positions[p-debut])
        elif p-debut >= nmax and not iste_cercles[j].get_visible():
            # on est déjà sur ceux qui sont cachés
            break
        else:
            liste_cercles[j].set_visible(False)
    mettre_a_jour_droite(equation_droite)
            
# L'axe pour changer l'étape
axcolor = 'lightgoldenrodyellow'
axfreq = plt.axes([0.2, 0.075, 0.6, 0.03], facecolor=axcolor)
sfreq = Slider(axfreq, 'etape', 0, t_max, valinit=0, valstep=1)
sfreq.on_changed(mise_a_jour)

# L'axe pour changer le début
axfreq2 = plt.axes([0.2, 0.04, 0.6, 0.03], facecolor=axcolor)
sfreq2 = Slider(axfreq2, 'debut', 1, debut_max, valinit=DEBUT, valstep=1)
sfreq2.on_changed(mise_a_jour_debut)

# La zone de texte pour afficher l'équation de droite
axbox = plt.axes([0.2, 0.9, 0.6, 0.05])
text_box = TextBox(axbox, 'U(n) = ', initial=equation_droite)
text_box.on_submit(mettre_a_jour_droite)

# On met tout en place pour le premier affichage
mise_a_jour(0)
mise_a_jour_debut(DEBUT)

plt.show()